use nmea::Nmea;

const MAX_SENTENCE_LEN: usize = 128;

#[derive(Copy, Clone)]
pub struct GpsPos {
    pub latitude: f64,
    pub longitude: f64,
    pub altitude_meters: f32,
    pub speed_ms: f32,
    pub true_course: f32,
}

pub struct GpsModule {
    nmea: Nmea,
    cached_pos: Option<GpsPos>,

    sentence: [u8; MAX_SENTENCE_LEN],
    sentence_i: usize,
}

impl GpsModule {
    pub fn new() -> Self {
        Self {
            nmea: Nmea::default(),
            cached_pos: None,

            sentence: [0; MAX_SENTENCE_LEN],
            sentence_i: 0,
        }
    }

    pub fn consume(&mut self, b: u8) {
        if b == b'\n' {
            // end of sentence

            // only process sentences that were less than the max length
            // (and therefore were not truncated)
            if self.sentence_i <= MAX_SENTENCE_LEN {
                if let Ok(sentence) = core::str::from_utf8(&self.sentence[..self.sentence_i]) {
                    if self.nmea.parse(sentence).is_ok() {
                        if let Some(pos) = self.raw_pos() {
                            self.cached_pos = Some(pos);
                        }
                    }
                }
            }

            self.sentence_i = 0;
        } else {
            if self.sentence_i < MAX_SENTENCE_LEN {
                self.sentence[self.sentence_i] = b;
            }

            self.sentence_i += 1;
        }
    }

    pub fn pos(&self) -> Option<GpsPos> {
        self.cached_pos
    }

    fn raw_pos(&self) -> Option<GpsPos> {
        let x = GpsPos {
            latitude: self.nmea.latitude?,
            longitude: self.nmea.longitude?,
            altitude_meters: self.nmea.altitude?,
            speed_ms: self.nmea.speed_over_ground? * 0.514_444_5,
            true_course: self.nmea.true_course?,
        };

        Some(x)
    }
}

impl Default for GpsModule {
    fn default() -> Self {
        Self::new()
    }
}
